-- FastTravel for X-Plane 2018.05.31 http://koredekaiketsu.blogspot.com/

local ffi = require("ffi")
local XPLM = nil
local XPLMlib = "XPLM"
if SYSTEM_ARCHITECTURE == 64 then
	if SYSTEM == "IBM" then
		XPLMlib = "XPLM_64"
	elseif SYSTEM == "LIN" then
		XPLMlib = "Resources/plugins/XPLM_64.so"
	else
		XPLMlib = "Resources/plugins/XPLM.framework/XPLM"
	end
else
	if SYSTEM == "IBM" then
		XPLMlib = "XPLM"
	elseif SYSTEM == "LIN" then
		XPLMlib = "Resources/plugins/XPLM.so"
	else
		XPLMlib = "Resources/plugins/XPLM.framework/XPLM"
	end
end

XPLM = ffi.load(XPLMlib)
-- Define FlyWithLua wrapper for XPLM interfaces:
ffi.cdef[[
void XPLMLocalToWorld(
	double inX, double inY, double inZ, double *outLatitude, double *outLongitude, double *outAltitude);
]]
ffi.cdef[[
void XPLMWorldToLocal(
	double inLatitude, double inLongitude, double inAltitude, double *outX, double *outY, double *outZ);
]]

local function geo_distance(lat1, lon1, lat2, lon2)
	local sin_dlat = math.sin(math.rad(lat2 - lat1) / 2)
	local sin_dlon = math.sin(math.rad(lon2 - lon1) / 2)
	local a = sin_dlat * sin_dlat + math.cos(math.rad(lat1)) * math.cos(math.rad(lat2)) * sin_dlon * sin_dlon
	return 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a)) * 6378100
end

local lx = ffi.new("double[1]")
local ly = ffi.new("double[1]")
local lz = ffi.new("double[1]")
local rx = ffi.new("double[1]")
local ry = ffi.new("double[1]")
local rz = ffi.new("double[1]")

local speed = 1
local offset = false
local offsetX = 0
local offsetY = 0
local offsetZ = 0

function every_draw()

	if speed == 1 then
		return
	end

	local x = get("sim/flightmodel/position/local_x");
	local y = get("sim/flightmodel/position/local_y");
	local z = get("sim/flightmodel/position/local_z");

	if offset == false then
		set("sim/time/sim_speed", 0)
		XPLM.XPLMLocalToWorld(x, y, z, lx, ly, lz)
		offsetX = lx[0] - sx
		offsetY = ly[0] - sy
		if sz == -1 then
			offsetZ = 0
		else
			offsetZ = lz[0] - sz
		end
		offset = true
	end

	XPLM.XPLMLocalToWorld(x, y, z, lx, ly, lz)
	x = lx[0] + offsetX * speed
	y = ly[0] + offsetY * speed
	z = lz[0] + offsetZ * speed
	XPLM.XPLMWorldToLocal(x, y, z, lx, ly, lz)
	set("sim/flightmodel/position/local_x", lx[0])
	set("sim/flightmodel/position/local_y", ly[0])
	set("sim/flightmodel/position/local_z", lz[0])

	if groundspeed > 0 then
		set("sim/time/zulu_time_sec", startTime + geo_distance(sx, sy, x, y) / groundspeed)
	end

	local lt = get("sim/time/local_time_sec")

	draw_string(10, 10, "Speed="..speed.." Ground Speed="..groundspeed.." Time="..string.format("%02d", lt / 3600)..":"..string.format("%02d", lt % 3600 / 60)..":"..string.format("%02d", lt % 3600 % 60).." Altitude="..(z * 3.28084).." Latitude="..x.." Longitude="..y, "green")
end
do_every_draw("every_draw()")

function inclesase_speed(maintain_altitude, time_advance)

	if speed >= 256 then
		return
	end

	if speed == 1 then
		startTime = get("sim/time/zulu_time_sec")
		if time_advance == 1 then
			groundspeed = get("sim/flightmodel/position/groundspeed")
		else
			groundspeed = 0
		end

		local x = get("sim/flightmodel/position/local_x")
		local y = get("sim/flightmodel/position/local_y")
		local z = get("sim/flightmodel/position/local_z")
		XPLM.XPLMLocalToWorld(x, y, z, lx, ly, lz)
		sx = lx[0]
		sy = ly[0]
		if maintain_altitude == 1 then
			sz = -1
		else
			sz = lz[0]
		end
		offset = false
	end

	speed = speed * 2
end

function decrease_speed()
	if speed == 1 then
		return
	end

	speed = speed / 2

	if speed == 1 then
		set("sim/time/sim_speed", 1)
	end
end

function reset_speed()
	speed = 1
	set("sim/time/sim_speed", 1)
end

create_command("FlyWithLua/FastTravel/InclesaseSpeed", "FastTravel: Inclesase speed", "", "", "inclesase_speed(0, 0)")
create_command("FlyWithLua/FastTravel/InclesaseSpeedMaintainAltitude", "FastTravel: Inclesase speed(Maintain altitude)", "", "", "inclesase_speed(1, 0)")
create_command("FlyWithLua/FastTravel/InclesaseSpeedTimeAdvance", "FastTravel: Inclesase speed(Time advance)", "", "", "inclesase_speed(0, 1)")
create_command("FlyWithLua/FastTravel/InclesaseSpeedMaintainAltitudeTimeAdvance", "FastTravel: Inclesase speed(Maintain altitude, Time advance)", "", "", "inclesase_speed(1, 1)")
create_command("FlyWithLua/FastTravel/DecreaseSpeed", "FastTravel: Decrease speed", "", "", "decrease_speed()")
create_command("FlyWithLua/FastTravel/ResetSpeed", "FastTravel: Reset speed", "", "", "reset_speed()")

